#!/bin/bash

######################################################################
# Variables
######################################################################
xteamSlaveFile='files/XTEAMSlave.tar.gz'

logFileName='create_log'

sleepDuration=60

######################################################################
# Import Variables
######################################################################
source variables.sh

######################################################################
# Create Instances
######################################################################
echo
echo 'Beginning to create instances ...'
gcloud compute instances create $nodes --zone $farmZone --image $imageID --machine-type $machineType

echo 'GCE instance creation completed.'

echo
echo 'Sleeping for' $sleepDuration 'seconds to wait for the instances to get ready ...'
sleep $sleepDuration

######################################################################
# Push Files
######################################################################
echo
echo 'Beginning to push files to instances ...'
for node in $nodes
do
	echo 'Copying installation package to' $node '...'
	gcloud compute copy-files files/XTEAMSlave.tar.gz $node:~ --zone $farmZone -q 
	echo 'Extracting installation package on' $node '...'
	gcloud compute ssh $node --zone $farmZone --command "tar zxvf ./XTEAMSlave.tar.gz >& extract.log.$node" -q &> $logFileName
	echo 'Generating the slave name file on' $node '...'
	gcloud compute ssh $node --zone $farmZone --command "echo $node > node_name.txt" -q &> $logFileName
	echo 'Installing libraries on' $node '...'
	gcloud compute ssh $node --zone $farmZone --command "./install.sh >& install.log.$node" -q &> $logFileName &
done

echo 'GCE instance initialization completed.'
